@@ -90,6 +90,7 @@ public:
     QUrl launcherUrl(WId window, bool encodeFallbackIcon = true);
     QUrl serviceUrl(int pid, const QString &type, const QStringList &cmdRemovals);
     KService::List servicesFromPid(int pid);
+    KService::List servicesFromCmdLine(const QString &cmdLine, const QString &processName);
     bool demandsAttention(WId window);
 
 private:
@@ -517,6 +518,7 @@ QUrl XWindowTasksModel::Private::windowUrl(WId window)
         KConfigGroup grp(rulesConfig, "Mapping");
         KConfigGroup set(rulesConfig, "Settings");
 
+        // Evaluate MatchCommandLineFirst directives from config first.
         // Some apps have different launchers depending upon command line ...
         QStringList matchCommandLineFirst = set.readEntry("MatchCommandLineFirst", QStringList());
 
@@ -531,15 +533,15 @@ QUrl XWindowTasksModel::Private::windowUrl(WId window)
             services = servicesFromPid(pid);
         }
 
-        // If the user has manually set a mapping, respect this first...
-        QString mapped(grp.readEntry(classClass + "::" + className, QString()));
-
-        if (mapped.endsWith(QLatin1String(".desktop"))) {
-            url = QUrl(mapped);
-            return url;
-        }
-
         if (!classClass.isEmpty()) {
+            // Evaluate any mapping rules that map to a specific .desktop file.
+            QString mapped(grp.readEntry(classClass + "::" + className, QString()));
+
+            if (mapped.endsWith(QLatin1String(".desktop"))) {
+                url = QUrl(mapped);
+                return url;
+            }
+
             if (mapped.isEmpty()) {
                 mapped = grp.readEntry(classClass, QString());
 
@@ -557,75 +559,95 @@ QUrl XWindowTasksModel::Private::windowUrl(WId window)
                 return url;
             }
 
-            KConfigGroup rewriteRulesGroup(rulesConfig, QStringLiteral("Rewrite Rules"));
-            if (rewriteRulesGroup.hasGroup(classClass)) {
-                KConfigGroup rewriteGroup(&rewriteRulesGroup, classClass);
+            // Try matching both WM_CLASS instance and general class against StartupWMClass.
+            // We do this before evaluating the mapping rules further, because StartupWMClass
+            // is essentially a mapping rule, and we expect it to be set deliberately and
+            // sensibly to instruct us what to do. Also, mapping rules
+            //
+            // StartupWMClass=STRING
+            //
+            //   If true, it is KNOWN that the application will map at least one
+            //   window with the given string as its WM class or WM name hint.
+            //
+            // Source: https://specifications.freedesktop.org/startup-notification-spec/startup-notification-0.1.txt
+            if (services.empty()) {
+                services = KServiceTypeTrader::self()->query(QStringLiteral("Application"), QStringLiteral("exist Exec and ('%1' =~ StartupWMClass)").arg(classClass));
+            }
 
-                const QStringList &rules = rewriteGroup.groupList();
-                for (const QString &rule : rules) {
-                    KConfigGroup ruleGroup(&rewriteGroup, rule);
+            if (services.empty()) {
+                services = KServiceTypeTrader::self()->query(QStringLiteral("Application"), QStringLiteral("exist Exec and ('%1' =~ StartupWMClass)").arg(className));
+            }
 
-                    const QString propertyConfig = ruleGroup.readEntry(QStringLiteral("Property"), QString());
+            // Evaluate rewrite rules from config.
+            if (services.empty()) {
+                KConfigGroup rewriteRulesGroup(rulesConfig, QStringLiteral("Rewrite Rules"));
+                if (rewriteRulesGroup.hasGroup(classClass)) {
+                    KConfigGroup rewriteGroup(&rewriteRulesGroup, classClass);
 
-                    QString matchProperty;
-                    if (propertyConfig == QLatin1String("ClassClass")) {
-                        matchProperty = classClass;
-                    } else if (propertyConfig == QLatin1String("ClassName")) {
-                        matchProperty = className;
-                    }
+                    const QStringList &rules = rewriteGroup.groupList();
+                    for (const QString &rule : rules) {
+                        KConfigGroup ruleGroup(&rewriteGroup, rule);
 
-                    if (matchProperty.isEmpty()) {
-                        continue;
-                    }
+                        const QString propertyConfig = ruleGroup.readEntry(QStringLiteral("Property"), QString());
 
-                    const QString serviceSearchIdentifier = ruleGroup.readEntry(QStringLiteral("Identifier"), QString());
-                    if (serviceSearchIdentifier.isEmpty()) {
-                        continue;
-                    }
+                        QString matchProperty;
+                        if (propertyConfig == QLatin1String("ClassClass")) {
+                            matchProperty = classClass;
+                        } else if (propertyConfig == QLatin1String("ClassName")) {
+                            matchProperty = className;
+                        }
 
-                    QRegularExpression regExp(ruleGroup.readEntry(QStringLiteral("Match")));
-                    const auto match = regExp.match(matchProperty);
-
-                    if (match.hasMatch()) {
-                        const QString actualMatch = match.captured(QStringLiteral("match"));
-                        if (actualMatch.isEmpty()) {
+                        if (matchProperty.isEmpty()) {
                             continue;
                         }
 
-                        QString rewrittenString = ruleGroup.readEntry(QStringLiteral("Target")).arg(actualMatch);
-                        // If no "Target" is provided, instead assume the matched property (ClassClass/ClassName).
-                        if (rewrittenString.isEmpty()) {
-                            rewrittenString = matchProperty;
+                        const QString serviceSearchIdentifier = ruleGroup.readEntry(QStringLiteral("Identifier"), QString());
+                        if (serviceSearchIdentifier.isEmpty()) {
+                            continue;
                         }
 
-                        services = KServiceTypeTrader::self()->query(QStringLiteral("Application"), QStringLiteral("exist Exec and ('%1' =~ %2)").arg(rewrittenString, serviceSearchIdentifier));
+                        QRegularExpression regExp(ruleGroup.readEntry(QStringLiteral("Match")));
+                        const auto match = regExp.match(matchProperty);
 
-                        if (!services.isEmpty()) {
-                            break;
+                        if (match.hasMatch()) {
+                            const QString actualMatch = match.captured(QStringLiteral("match"));
+                            if (actualMatch.isEmpty()) {
+                                continue;
+                            }
+
+                            QString rewrittenString = ruleGroup.readEntry(QStringLiteral("Target")).arg(actualMatch);
+                            // If no "Target" is provided, instead assume the matched property (ClassClass/ClassName).
+                            if (rewrittenString.isEmpty()) {
+                                rewrittenString = matchProperty;
+                            }
+
+                            services = KServiceTypeTrader::self()->query(QStringLiteral("Application"), QStringLiteral("exist Exec and ('%1' =~ %2)").arg(rewrittenString, serviceSearchIdentifier));
+
+                            if (!services.isEmpty()) {
+                                break;
+                            }
                         }
                     }
                 }
             }
 
+            // Try matching mapped name against DesktopEntryName.
             if (!mapped.isEmpty() && services.empty()) {
                 services = KServiceTypeTrader::self()->query(QStringLiteral("Application"), QStringLiteral("exist Exec and ('%1' =~ DesktopEntryName)").arg(mapped));
             }
 
+            // Try matching mapped name against 'Name'.
             if (!mapped.isEmpty() && services.empty()) {
                 services = KServiceTypeTrader::self()->query(QStringLiteral("Application"), QStringLiteral("exist Exec and ('%1' =~ Name) and (not exist NoDisplay or not NoDisplay)").arg(mapped));
             }
 
-            // To match other docks (docky, unity, etc.) attempt to match on DesktopEntryName first ...
+            // Try matching WM_CLASS general class against DesktopEntryName.
             if (services.empty()) {
                 services = KServiceTypeTrader::self()->query(QStringLiteral("Application"), QStringLiteral("exist Exec and ('%1' =~ DesktopEntryName)").arg(classClass));
             }
 
-            // Try StartupWMClass.
-            if (services.empty()) {
-                services = KServiceTypeTrader::self()->query(QStringLiteral("Application"), QStringLiteral("exist Exec and ('%1' =~ StartupWMClass)").arg(classClass));
-            }
-
-            // Try 'Name' - unfortunately this can be translated, so has a good chance of failing! (As it does for KDE's own "System Settings" (even in English!!))
+            // Try matching WM_CLASS general class against 'Name'.
+            // This has a shaky chance of success as WM_CLASS is untranslated, but 'Name' may be localized.
             if (services.empty()) {
                 services = KServiceTypeTrader::self()->query(QStringLiteral("Application"), QStringLiteral("exist Exec and ('%1' =~ Name) and (not exist NoDisplay or not NoDisplay)").arg(classClass));
             }
@@ -767,49 +789,77 @@ QUrl XWindowTasksModel::Private::serviceUrl(int pid, const QString &type, const
 
 KService::List XWindowTasksModel::Private::servicesFromPid(int pid)
 {
-    // Attempt to find using commandline...
-    KService::List services;
-
     if (pid == 0) {
-        return services;
+        return KService::List();
     }
 
     KSysGuard::Processes procs;
     procs.updateOrAddProcess(pid);
 
     KSysGuard::Process *proc = procs.getProcess(pid);
-    QString cmdline = proc ? proc->command().simplified() : QString(); // proc->command has a trailing space???
+    const QString &cmdLine = proc ? proc->command().simplified() : QString(); // proc->command has a trailing space???
 
-    if (cmdline.isEmpty()) {
-        return services;
+    if (cmdLine.isEmpty()) {
+        return KService::List();
     }
 
-    const int firstSpace = cmdline.indexOf(' ');
+    return servicesFromCmdLine(cmdLine, proc->name());
+}
+
+KService::List XWindowTasksModel::Private::servicesFromCmdLine(const QString &_cmdLine, const QString &processName)
+{
+    QString cmdLine = _cmdLine;
+    KService::List services;
+
+    const int firstSpace = cmdLine.indexOf(' ');
+    int slash = 0;
+
+    services = KServiceTypeTrader::self()->query(QStringLiteral("Application"), QStringLiteral("exist Exec and ('%1' =~ Exec)").arg(cmdLine));
 
-    services = KServiceTypeTrader::self()->query(QStringLiteral("Application"), QStringLiteral("exist Exec and ('%1' =~ Exec)").arg(cmdline));
     if (services.empty()) {
-        // Could not find with complete commandline, so strip out path part...
-        int slash = cmdline.lastIndexOf('/', firstSpace);
+        // Could not find with complete command line, so strip out the path part ...
+        slash = cmdLine.lastIndexOf('/', firstSpace);
+
         if (slash > 0) {
-            services = KServiceTypeTrader::self()->query(QStringLiteral("Application"), QStringLiteral("exist Exec and ('%1' =~ Exec)").arg(cmdline.mid(slash + 1)));
+            services = KServiceTypeTrader::self()->query(QStringLiteral("Application"), QStringLiteral("exist Exec and ('%1' =~ Exec)").arg(cmdLine.mid(slash + 1)));
         }
     }
 
     if (services.empty() && firstSpace > 0) {
-        // Could not find with arguments, so try without...
-        cmdline = cmdline.left(firstSpace);
-        services = KServiceTypeTrader::self()->query(QStringLiteral("Application"), QStringLiteral("exist Exec and ('%1' =~ Exec)").arg(cmdline));
+        // Could not find with arguments, so try without ...
+        cmdLine = cmdLine.left(firstSpace);
 
-        int slash = cmdline.lastIndexOf('/');
-        if (slash > 0) {
-            services = KServiceTypeTrader::self()->query(QStringLiteral("Application"), QStringLiteral("exist Exec and ('%1' =~ Exec)").arg(cmdline.mid(slash + 1)));
+        services = KServiceTypeTrader::self()->query(QStringLiteral("Application"), QStringLiteral("exist Exec and ('%1' =~ Exec)").arg(cmdLine));
+
+        if (services.empty()) {
+            slash = cmdLine.lastIndexOf('/');
+
+            if (slash > 0) {
+                services = KServiceTypeTrader::self()->query(QStringLiteral("Application"), QStringLiteral("exist Exec and ('%1' =~ Exec)").arg(cmdLine.mid(slash + 1)));
+            }
         }
     }
 
-    if (services.empty() && proc && !QStandardPaths::findExecutable(cmdline).isEmpty()) {
-        // cmdline now exists without arguments if there were any
-        services << QExplicitlySharedDataPointer<KService>(new KService(proc->name(), cmdline, QString()));
+    if (services.empty()) {
+        KConfigGroup set(rulesConfig, "Settings");
+        const QStringList &runtimes = set.readEntry("TryIgnoreRuntimes", QStringList());
+
+        bool ignore = runtimes.contains(cmdLine);
+
+        if (!ignore && slash > 0) {
+            ignore = runtimes.contains(cmdLine.mid(slash + 1));
+        }
+
+        if (ignore) {
+            return servicesFromCmdLine(_cmdLine.mid(firstSpace + 1), processName);
+        }
     }
+
+    if (services.empty() && !processName.isEmpty() && !QStandardPaths::findExecutable(cmdLine).isEmpty()) {
+        // cmdLine now exists without arguments if there were any.
+        services << QExplicitlySharedDataPointer<KService>(new KService(processName, cmdLine, QString()));
+    }
+
     return services;
 }
 
