@@ -19,24 +19,27 @@
 
 #include "recentdocuments.h"
 
+#include <QAction>
+#include <QDir>
 #include <QMimeData>
 
 #include <KDesktopFile>
 #include <KConfigGroup>
-#include <QDebug>
 #include <KDirWatch>
 #include <KRun>
 #include <KRecentDocument>
 #include <KLocalizedString>
+#include <KIO/OpenFileManagerWindowJob>
 
 K_EXPORT_PLASMA_RUNNER(recentdocuments, RecentDocuments)
 
+static const QString s_openParentDirId = QStringLiteral("openParentDir");
+
 RecentDocuments::RecentDocuments(QObject *parent, const QVariantList& args)
     : Plasma::AbstractRunner(parent, args)
 {
     Q_UNUSED(args);
     setObjectName( QStringLiteral("Recent Documents" ));
-    m_icon = QIcon::fromTheme(QStringLiteral("document-open-recent"));
     loadRecentDocuments();
     // listen for changes to the list of recent documents
     KDirWatch *recentDocWatch = new KDirWatch(this);
@@ -45,6 +48,8 @@ RecentDocuments::RecentDocuments(QObject *parent, const QVariantList& args)
     connect(recentDocWatch, &KDirWatch::deleted, this, &RecentDocuments::loadRecentDocuments);
     connect(recentDocWatch, &KDirWatch::dirty, this, &RecentDocuments::loadRecentDocuments);
     addSyntax(Plasma::RunnerSyntax(QStringLiteral(":q:"), i18n("Looks for documents recently used with names matching :q:.")));
+
+    addAction(s_openParentDirId, QIcon::fromTheme(QStringLiteral("document-open-folder")), i18n("Open Containing Folder"));
 }
 
 RecentDocuments::~RecentDocuments()
@@ -53,7 +58,6 @@ RecentDocuments::~RecentDocuments()
 
 void RecentDocuments::loadRecentDocuments()
 {
-    //qDebug() << "Refreshing recent documents.";
     m_recentdocuments = KRecentDocument::recentDocuments();
 }
 
@@ -69,6 +73,8 @@ void RecentDocuments::match(Plasma::RunnerContext &context)
         return;
     }
 
+    const QString homePath = QDir::homePath();
+
     foreach (const QString &document, m_recentdocuments) {
         if (!context.isValid()) {
             return;
@@ -79,10 +85,21 @@ void RecentDocuments::match(Plasma::RunnerContext &context)
             Plasma::QueryMatch match(this);
             match.setType(Plasma::QueryMatch::PossibleMatch);
             match.setRelevance(1.0);
-            match.setIcon(QIcon::fromTheme(config.readIcon()));
+            match.setIconName(config.readIcon());
             match.setData(config.readUrl());
             match.setText(config.readName());
-            match.setSubtext(i18n("Recent Document"));
+
+            QUrl folderUrl = QUrl(config.readUrl()).adjusted(QUrl::RemoveFilename | QUrl::StripTrailingSlash);
+            if (folderUrl.isLocalFile()) {
+                QString folderPath = folderUrl.toLocalFile();
+                if (folderPath.startsWith(homePath)) {
+                    folderPath.replace(0, homePath.length(), QStringLiteral("~"));
+                }
+                match.setSubtext(folderPath);
+            } else {
+                match.setSubtext(folderUrl.toDisplayString());
+            }
+
             context.addMatch(match);
         }
     }
@@ -91,18 +108,33 @@ void RecentDocuments::match(Plasma::RunnerContext &context)
 void RecentDocuments::run(const Plasma::RunnerContext &context, const Plasma::QueryMatch &match)
 {
     Q_UNUSED(context)
-    QString url = match.data().toString();
-    qDebug() << "Opening Recent Document" << url;
-    new KRun(url, 0);
+
+    const QString url = match.data().toString();
+
+    if (match.selectedAction() && match.selectedAction() == action(s_openParentDirId)) {
+        KIO::highlightInFileManager({QUrl(url)});
+        return;
+    }
+
+    new KRun(QUrl(url), 0);
+}
+
+QList<QAction *> RecentDocuments::actionsForMatch(const Plasma::QueryMatch &match)
+{
+    Q_UNUSED(match)
+
+    QList<QAction *> actions;
+
+    if (QUrl(match.data().toString()).isLocalFile()) {
+        actions << action(s_openParentDirId);
+    }
+
+    return actions;
 }
 
 QMimeData * RecentDocuments::mimeDataForMatch(const Plasma::QueryMatch& match)
 {
-    QMimeData * result = new QMimeData();
-    QList<QUrl> urls;
-    urls << QUrl(match.data().toString());
-    result->setUrls(urls);
-
+    QMimeData *result = new QMimeData();
     result->setText(match.data().toString());
     return result;
 }
