@@ -209,7 +209,7 @@ SNIProxy::SNIProxy(xcb_window_t wid, QObject* parent):
     //if the client does supports that we send directly, otherwise we'll use xtest
     auto waCookie = xcb_get_window_attributes(c, wid);
     auto windowAttributes = xcb_get_window_attributes_reply(c, waCookie, nullptr);
-    if (! windowAttributes->all_event_masks & XCB_EVENT_MASK_BUTTON_PRESS) {
+    if (! (windowAttributes->all_event_masks & XCB_EVENT_MASK_BUTTON_PRESS)) {
         m_injectMode = XTest;
     }
 
